#version 330 core

const int maxNumberOfLights = 4;

struct LightSource {
	vec3 position; //Position in worldspace
	vec3 color;
	vec3 spotDirection;
	//attenuation
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;

	int on; //0 if off, 1 if on
	int isSpot; //1 if spotlight
	float spotCutoff; 
	float spotExponent;
};

layout(location = 0) in vec3 position; //in model space
layout(location = 1) in vec3 normal; 
layout(location = 2) in vec3 tangent;
layout(location = 3) in vec3 bitangent;
layout(location = 4) in vec2 uv; //texture coordinates

out vec2 fragmentUV;
out vec3 position_worldspace;
out vec3 position_cameraspace; // Position of the Vertex in cameraspace
out vec4 shadowCoordinate; //Coordinate for shadow mapping

out vec3 Normal_cameraspace;
out vec3 LightDirection_tangentspace[maxNumberOfLights];
out vec3 EyeDirection_tangentspace;

uniform mat4 MVP;
uniform mat4 modelViewMatrix;
uniform mat4 depthMatrix;
uniform mat3 MVi;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform LightSource lights[maxNumberOfLights];
uniform int numberOfLights;

void main() 
{
	position_cameraspace = (modelViewMatrix * vec4(position,1.0)).xyz;
	position_worldspace = (modelMatrix * vec4(position,1.0)).xyz;

	// Transform normal,tangent and bitangent to cameraspace
	vec3 normal_cameraspace    = MVi * normal;
	vec3 tangent_cameraspace   = MVi * tangent;
	vec3 bitangent_cameraspace = MVi * bitangent;

	// Matrix for transformation to tangent space
	mat3 toObjectLocal = mat3(
		tangent_cameraspace.x, bitangent_cameraspace.x, normal_cameraspace.x,
		tangent_cameraspace.y, bitangent_cameraspace.y, normal_cameraspace.y,
		tangent_cameraspace.z, bitangent_cameraspace.z, normal_cameraspace.z ) ;

	//transform light direction and view direction to tangent space

	//For all lights
	for(int i=0; i < numberOfLights; i++) {
		LightDirection_tangentspace[i] = toObjectLocal * ((viewMatrix * vec4(lights[i].position,1.0)).xyz - position_cameraspace);
	}
	EyeDirection_tangentspace = toObjectLocal * -position_cameraspace;

	fragmentUV = uv;

	gl_Position = MVP * vec4(position, 1);
	shadowCoordinate = depthMatrix * vec4(position_worldspace,1.0);
}