#version 330

const int maxNumberOfLights = 4;

struct LightSource {
	vec3 position; //Position in worldspace
	vec3 color;
	vec3 spotDirection;
	//attenuation
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;

	int on; //0 if off, 1 if on
	int isSpot; //1 if spotlight
	float spotCutoff; 
	float spotExponent;
};

layout(location = 0) in vec3 position; 
layout(location = 1) in vec3 normal; 
layout(location = 2) in vec2 uv; //texture coordinates

out vec2 fragmentUV;
out vec3 position_worldspace;
out vec3 position_cameraspace; // Position of the Vertex in cameraspace
out vec4 shadowCoordinate; //Coordinate for shadow mapping

out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace[maxNumberOfLights];

uniform mat4 MVP;
uniform mat4 modelViewMatrix;
uniform mat4 depthMatrix;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat3 MVi;
uniform LightSource lights[maxNumberOfLights];
uniform int numberOfLights;

void main() 
{
	position_cameraspace = (modelViewMatrix * vec4(position,1.0)).xyz;
	position_worldspace = (modelMatrix * vec4(position,1.0)).xyz;

	Normal_cameraspace    = MVi * normal;
	EyeDirection_cameraspace    = normalize(-position_cameraspace);

	//For all lights
	for(int i=0; i < numberOfLights; i++) {
		LightDirection_cameraspace[i] = normalize( (viewMatrix * vec4(lights[i].position,1.0)).xyz - position_cameraspace);
	}

	fragmentUV = uv;
	gl_Position = MVP * vec4(position, 1);
	shadowCoordinate = depthMatrix * vec4(position_worldspace,1);
}